"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * @license
 * Copyright Lightweightform. All Rights Reserved.
 *
 * Use of this source code is governed by an Apache2.0-style license that can be
 * found at https://www.apache.org/licenses/LICENSE-2.0
 */
var utils_1 = require("../../utils");
var template_manager_1 = require("../template.manager");
var RecordStorage = /** @class */ (function () {
    function RecordStorage() {
    }
    RecordStorage.createRecordStorage = function (schema, basePackage, directory, name, isRoot, isList) {
        var _this = this;
        if (isRoot === void 0) { isRoot = false; }
        if (isList === void 0) { isList = false; }
        if (!isRoot) {
            // whether the record isn't root record creates the directory of that level
            utils_1.createDirectory(directory);
        }
        var fields = [];
        var importPackages = [];
        Object.keys(schema.fieldsSchemas).forEach(function (fieldName) {
            var fieldSchema = schema.fieldsSchemas[fieldName];
            var isRecordSchema = fieldSchema.type === 'record';
            var isListSchema = fieldSchema.type === 'list';
            var isPairSchema = fieldSchema.type === 'tuple';
            var isNullable = fieldSchema.isNullable;
            var isCheckboxGroupSchema = fieldSchema.elementsSchema &&
                (fieldSchema.elementsSchema.type === 'string' ||
                    fieldSchema.elementsSchema.type === 'number');
            var fieldType;
            switch (fieldSchema.type) {
                case 'record':
                    fieldType = utils_1.camelCase(fieldName);
                    RecordStorage.createRecordStorage(fieldSchema, basePackage + "." + fieldName.toLowerCase(), directory + "/" + fieldName.toLowerCase(), fieldName);
                    break;
                case 'list':
                    fieldType = utils_1.camelCase(fieldName) + "Entry";
                    var childSchema = fieldSchema.elementsSchema;
                    if (isCheckboxGroupSchema) {
                        fieldType = utils_1.camelCase(childSchema.type);
                    }
                    else {
                        RecordStorage.createRecordStorage(childSchema, basePackage + "." + fieldName.toLowerCase(), directory + "/" + fieldName.toLowerCase(), fieldType, false, true);
                    }
                    break;
                case 'tuple':
                    importPackages = _this._importPackageIfNotExists(importPackages, 'pt.lightweightform.kotlin.schema.PairSchema');
                    if (fieldSchema.elementsSchemas.length !== 2) {
                        throw new Error('LF-KOTLIN only support Pairs at this moment');
                    }
                    isPairSchema = true;
                    break;
                case 'number':
                    fieldType = 'Long';
                    break;
                default:
                    fieldType = utils_1.camelCase(fieldSchema.type.replace('Schema', ''));
                    if (fieldType === 'Date') {
                        fieldType = 'String';
                    }
                    break;
            }
            fields.push({
                fieldName: fieldName,
                fieldType: fieldType,
                isNullable: isNullable,
                isList: isListSchema,
                isPairSchema: isPairSchema,
                isRecord: isRecordSchema,
            });
            if (isRecordSchema || (isListSchema && !isCheckboxGroupSchema)) {
                importPackages = _this._importPackageIfNotExists(importPackages, basePackage + "." + fieldName.toLowerCase() + "." + fieldType);
            }
        });
        // Write Record Storage class
        var className = utils_1.camelCase(name);
        utils_1.writeFile(directory + "/" + className + ".kt", template_manager_1.default.loadRecordStorage({
            package: "" + basePackage,
            className: utils_1.camelCase(name),
            fields: fields,
            importPackages: importPackages,
        }));
    };
    RecordStorage._importPackageIfNotExists = function (importPackages, packageToImport) {
        if (importPackages.indexOf(packageToImport) === -1) {
            importPackages.push(packageToImport);
        }
        return importPackages;
    };
    return RecordStorage;
}());
exports.RecordStorage = RecordStorage;
//# sourceMappingURL=record.storage.js.map